// -------------------------------------------------------------
// Winsock constants and structures
// -------------------------------------------------------------

#define SOCKET_HANDLE32       uint
#define SOCKET_HANDLE64       ulong
#define AF_INET               2
#define SOCK_STREAM           1
#define IPPROTO_TCP           6
#define INVALID_SOCKET32      0xFFFFFFFF
#define INVALID_SOCKET64      0xFFFFFFFFFFFFFFFF
#define SOCKET_ERROR          -1
#define INADDR_NONE           0xFFFFFFFF
#define FIONBIO               0x8004667E
#define WSAWOULDBLOCK         10035

struct sockaddr
{
    short family;
    ushort port;
    uint address;
    ulong ignore;
};

struct linger
{
    ushort onoff;
    ushort linger_seconds;
};

class IWinSocketWrapper
{
public:
    IWinSocketWrapper();
    virtual ~IWinSocketWrapper();

    // --------------------------------- from ws2_32.dll
    // Imports for 32-bit environment
    // virtual SOCKET_HANDLE32 socket(int, int, int) const PURE(0); // Artificially differs from 64-bit version based on 3rd parameter
    virtual SOCKET_HANDLE32 w_socket(int, int, int); // Artificially differs from 64-bit version based on 3rd parameter
    virtual int w_connect(SOCKET_HANDLE32, sockaddr&, int);
    virtual int w_closesocket(SOCKET_HANDLE32);
    virtual int w_send(SOCKET_HANDLE32, uchar&[],int,int);
    virtual int w_recv(SOCKET_HANDLE32, uchar&[], int, int);
    virtual int w_ioctlsocket(SOCKET_HANDLE32, uint, uint&);
    virtual int w_bind(SOCKET_HANDLE32, sockaddr&, int);
    virtual int w_listen(SOCKET_HANDLE32, int);
    virtual SOCKET_HANDLE32 w_accept(SOCKET_HANDLE32, int, int);
    virtual int w_WSAAsyncSelect(SOCKET_HANDLE32, int, uint, int);
    virtual int w_shutdown(SOCKET_HANDLE32, int);

    // Imports for 64-bit environment
    virtual SOCKET_HANDLE64 w_socket(int, int, uint); // Artificially differs from 32-bit version based on 3rd parameter
    virtual int w_connect(SOCKET_HANDLE64, sockaddr&, int);
    virtual int w_closesocket(SOCKET_HANDLE64);
    virtual int w_send(SOCKET_HANDLE64, uchar&[], int, int);
    virtual int w_recv(SOCKET_HANDLE64, uchar&[], int, int);
    virtual int w_ioctlsocket(SOCKET_HANDLE64, uint, uint&);
    virtual int w_bind(SOCKET_HANDLE64, sockaddr&, int);
    virtual int w_listen(SOCKET_HANDLE64, int);
    virtual SOCKET_HANDLE64 w_accept(SOCKET_HANDLE64, int, int);
    virtual int w_WSAAsyncSelect(SOCKET_HANDLE64, long, uint, int);
    virtual int w_shutdown(SOCKET_HANDLE64, int);

    // gethostbyname() has to vary between 32/64-bit, because
    // it returns a memory pointer whose size will be either
    // 4 bytes or 8 bytes. In order to keep the compiler
    // happy, we therefore need versions which take
    // artificially-different parameters on 32/64-bit
    virtual uint w_gethostbyname(uchar&[]); // For 32-bit
    virtual ulong w_gethostbyname(char&[]); // For 64-bit

    // Neutral; no difference between 32-bit and 64-bit
    virtual uint w_inet_addr(uchar&[]);
    virtual int w_WSAGetLastError();
    virtual uint w_htonl(uint);
    virtual ushort w_htons(ushort);

    // --------------------------------- from kernel32.dll
    virtual void w_RtlMoveMemory(uint&, uint, int) ;
    virtual void w_RtlMoveMemory(ushort&, uint, int) ;
    virtual void w_RtlMoveMemory(ulong&, ulong, int) ;
    virtual void w_RtlMoveMemory(ushort&, ulong, int) ;
};

IWinSocketWrapper::IWinSocketWrapper() {}
IWinSocketWrapper::~IWinSocketWrapper() {}

SOCKET_HANDLE32 IWinSocketWrapper::w_socket(int a, int b, int c) // Artificially differs from 64-bit version based on 3rd parameter
{
    return 0;
}

int IWinSocketWrapper::w_connect(SOCKET_HANDLE32, sockaddr&, int)
{
    return 0;
}

int IWinSocketWrapper::w_closesocket(SOCKET_HANDLE32)
{
    return 0;
}

int IWinSocketWrapper::w_send(SOCKET_HANDLE32, uchar&[],int,int)
{
    return 0;
}

int IWinSocketWrapper::w_recv(SOCKET_HANDLE32, uchar&[], int, int)
{
    return 0;
}

int IWinSocketWrapper::w_ioctlsocket(SOCKET_HANDLE32, uint, uint&)
{
    return 0;
}

int IWinSocketWrapper::w_bind(SOCKET_HANDLE32, sockaddr&, int)
{
    return 0;
}

int IWinSocketWrapper::w_listen(SOCKET_HANDLE32, int)
{
    return 0;
}

SOCKET_HANDLE32 IWinSocketWrapper::w_accept(SOCKET_HANDLE32, int, int)
{
    return 0;
}

int IWinSocketWrapper::w_WSAAsyncSelect(SOCKET_HANDLE32, int, uint, int)
{
    return 0;
}

int IWinSocketWrapper::w_shutdown(SOCKET_HANDLE32, int)
{
    return 0;
}


SOCKET_HANDLE64 IWinSocketWrapper::w_socket(int, int, uint) // Artificially differs from 32-bit version based on 3rd parameter
{
    return 0;
}

int IWinSocketWrapper::w_connect(SOCKET_HANDLE64, sockaddr&, int)
{
    return 0;
}

int IWinSocketWrapper::w_closesocket(SOCKET_HANDLE64)
{
    return 0;
}

int IWinSocketWrapper::w_send(SOCKET_HANDLE64, uchar&[], int, int)
{
    return 0;
}

int IWinSocketWrapper::w_recv(SOCKET_HANDLE64, uchar&[], int, int)
{
    return 0;
}

int IWinSocketWrapper::w_ioctlsocket(SOCKET_HANDLE64, uint, uint&)
{
    return 0;
}

int IWinSocketWrapper::w_bind(SOCKET_HANDLE64, sockaddr&, int)
{
    return 0;
}

int IWinSocketWrapper::w_listen(SOCKET_HANDLE64, int)
{
    return 0;
}

SOCKET_HANDLE64 IWinSocketWrapper::w_accept(SOCKET_HANDLE64, int, int)
{
    return 0;
}

int IWinSocketWrapper::w_WSAAsyncSelect(SOCKET_HANDLE64, long, uint, int)
{
    return 0;
}

int IWinSocketWrapper::w_shutdown(SOCKET_HANDLE64, int)
{
    return 0;
}


uint IWinSocketWrapper::w_gethostbyname(uchar&[]) // For 32-bit
{
    return 0;
}

ulong IWinSocketWrapper::w_gethostbyname(char&[]) // For 64-bit
{
    return 0;
}


uint IWinSocketWrapper::w_inet_addr(uchar&[])
{
    return 0;
}

int IWinSocketWrapper::w_WSAGetLastError()
{
    return 0;
}

uint IWinSocketWrapper::w_htonl(uint)
{
    return 0;
}

ushort IWinSocketWrapper::w_htons(ushort)
{
    return 0;
}

void IWinSocketWrapper::w_RtlMoveMemory(uint&, uint, int)
{
    return;
}

void IWinSocketWrapper::w_RtlMoveMemory(ushort&, uint, int)
{
    return;
}

void IWinSocketWrapper::w_RtlMoveMemory(ulong&, ulong, int)
{
    return;
}

void IWinSocketWrapper::w_RtlMoveMemory(ushort&, ulong, int)
{
    return;
}

