#include <DCLib/Network/IWinSocketWrapper.mqh>

#property strict

// -------------------------------------------------------------
// DLL imports
// -------------------------------------------------------------

#import "ws2_32.dll"
// Imports for 32-bit environment
SOCKET_HANDLE32 socket(int, int, int); // Artificially differs from 64-bit version based on 3rd parameter
int connect(SOCKET_HANDLE32, sockaddr&, int);
int closesocket(SOCKET_HANDLE32);
int send(SOCKET_HANDLE32, uchar&[],int,int);
int recv(SOCKET_HANDLE32, uchar&[], int, int);
int ioctlsocket(SOCKET_HANDLE32, uint, uint&);
int bind(SOCKET_HANDLE32, sockaddr&, int);
int listen(SOCKET_HANDLE32, int);
SOCKET_HANDLE32 accept(SOCKET_HANDLE32, int, int);
int WSAAsyncSelect(SOCKET_HANDLE32, int, uint, int);
int shutdown(SOCKET_HANDLE32, int);

// Imports for 64-bit environment
SOCKET_HANDLE64 socket(int, int, uint); // Artificially differs from 32-bit version based on 3rd parameter
int connect(SOCKET_HANDLE64, sockaddr&, int);
int closesocket(SOCKET_HANDLE64);
int send(SOCKET_HANDLE64, uchar&[], int, int);
int recv(SOCKET_HANDLE64, uchar&[], int, int);
int ioctlsocket(SOCKET_HANDLE64, uint, uint&);
int bind(SOCKET_HANDLE64, sockaddr&, int);
int listen(SOCKET_HANDLE64, int);
SOCKET_HANDLE64 accept(SOCKET_HANDLE64, int, int);
int WSAAsyncSelect(SOCKET_HANDLE64, long, uint, int);
int shutdown(SOCKET_HANDLE64, int);

// gethostbyname() has to vary between 32/64-bit, because
// it returns a memory pointer whose size will be either
// 4 bytes or 8 bytes. In order to keep the compiler
// happy, we therefore need versions which take
// artificially-different parameters on 32/64-bit
uint gethostbyname(uchar&[]); // For 32-bit
ulong gethostbyname(char&[]); // For 64-bit

// Neutral; no difference between 32-bit and 64-bit
uint inet_addr(uchar&[]);
int WSAGetLastError();
uint htonl(uint);
ushort htons(ushort);
#import

// For navigating the Winsock hostent structure, with indescribably horrible
// variation between 32-bit and 64-bit
#import "kernel32.dll"
void RtlMoveMemory(uint&, uint, int);
void RtlMoveMemory(ushort&, uint, int);
void RtlMoveMemory(ulong&, ulong, int);
void RtlMoveMemory(ushort&, ulong, int);
#import


class WinSocketWrapper : public IWinSocketWrapper
{
public:
    WinSocketWrapper();
    ~WinSocketWrapper();

    // --------------------------------- from ws2_32.dll
    // Imports for 32-bit environment
    SOCKET_HANDLE32 w_socket(int, int, int); // Artificially differs from 64-bit version based on 3rd parameter
    int w_connect(SOCKET_HANDLE32, sockaddr&, int);
    int w_closesocket(SOCKET_HANDLE32);
    int w_send(SOCKET_HANDLE32, uchar&[],int,int);
    int w_recv(SOCKET_HANDLE32, uchar&[], int, int);
    int w_ioctlsocket(SOCKET_HANDLE32, uint, uint&);
    int w_bind(SOCKET_HANDLE32, sockaddr&, int);
    int w_listen(SOCKET_HANDLE32, int);
    SOCKET_HANDLE32 w_accept(SOCKET_HANDLE32, int, int);
    int w_WSAAsyncSelect(SOCKET_HANDLE32, int, uint, int);
    int w_shutdown(SOCKET_HANDLE32, int);

    // Imports for 64-bit environment
    SOCKET_HANDLE64 w_socket(int, int, uint); // Artificially differs from 32-bit version based on 3rd parameter
    int w_connect(SOCKET_HANDLE64, sockaddr&, int);
    int w_closesocket(SOCKET_HANDLE64);
    int w_send(SOCKET_HANDLE64, uchar&[], int, int);
    int w_recv(SOCKET_HANDLE64, uchar&[], int, int);
    int w_ioctlsocket(SOCKET_HANDLE64, uint, uint&);
    int w_bind(SOCKET_HANDLE64, sockaddr&, int);
    int w_listen(SOCKET_HANDLE64, int);
    SOCKET_HANDLE64 w_accept(SOCKET_HANDLE64, int, int);
    int w_WSAAsyncSelect(SOCKET_HANDLE64, long, uint, int);
    int w_shutdown(SOCKET_HANDLE64, int);

    // gethostbyname() has to vary between 32/64-bit, because
    // it returns a memory pointer whose size will be either
    // 4 bytes or 8 bytes. In order to keep the compiler
    // happy, we therefore need versions which take
    // artificially-different parameters on 32/64-bit
    uint w_gethostbyname(uchar&[]); // For 32-bit
    ulong w_gethostbyname(char&[]); // For 64-bit

    // Neutral; no difference between 32-bit and 64-bit
    uint w_inet_addr(uchar&[]);
    int w_WSAGetLastError();
    uint w_htonl(uint);
    ushort w_htons(ushort);

    // --------------------------------- from kernel32.dll
    void w_RtlMoveMemory(uint&, uint, int);
    void w_RtlMoveMemory(ushort&, uint, int);
    void w_RtlMoveMemory(ulong&, ulong, int);
    void w_RtlMoveMemory(ushort&, ulong, int);
};

WinSocketWrapper::WinSocketWrapper() {}
WinSocketWrapper::~WinSocketWrapper() {}

SOCKET_HANDLE32 WinSocketWrapper::w_socket(int family, int socket_type, int proto)
{
    return socket(family, socket_type, proto);
}

int WinSocketWrapper::w_connect(SOCKET_HANDLE32 socket_hd, sockaddr& server, int size_of_address)
{
    return connect(socket_hd, server, size_of_address);
}

int WinSocketWrapper::w_closesocket(SOCKET_HANDLE32 socket_hd)
{
    return closesocket(socket_hd);
}

int WinSocketWrapper::w_send(SOCKET_HANDLE32 socket_hd, uchar& arr[], int szToSend, int dd)
{
    return send(socket_hd, arr, szToSend, dd);
}

int WinSocketWrapper::w_recv(SOCKET_HANDLE32 socket_hd, uchar& arrBuffer[], int ReceiveBufferSize, int dd)
{
    return recv(socket_hd, arrBuffer, ReceiveBufferSize, dd);
}

int WinSocketWrapper::w_ioctlsocket(SOCKET_HANDLE32 socket_hd, uint fio, uint& is_non_block)
{
    return ioctlsocket(socket_hd, fio, is_non_block);
}

int WinSocketWrapper::w_bind(SOCKET_HANDLE32 socket_hd, sockaddr& server, int size_of_address)
{
    return bind(socket_hd, server, size_of_address);
}

int WinSocketWrapper::w_listen(SOCKET_HANDLE32 socket_hd, int client_count)
{
    return listen(socket_hd, client_count);
}

SOCKET_HANDLE32 WinSocketWrapper::w_accept(SOCKET_HANDLE32 socket_hd, int dd_1, int dd_2)
{
    return accept(socket_hd, dd_1, dd_2);
}

int WinSocketWrapper::w_WSAAsyncSelect(SOCKET_HANDLE32 socket_hd, int win_hd, uint key, int event)
{
    return WSAAsyncSelect(socket_hd, win_hd, key, event);
}

int WinSocketWrapper::w_shutdown(SOCKET_HANDLE32 socket_hd, int dd)
{
    return shutdown(socket_hd, dd);
}

//
SOCKET_HANDLE64 WinSocketWrapper::w_socket(int family, int socket_type, uint proto)
{
    return socket(family, socket_type, proto);
}

int WinSocketWrapper::w_connect(SOCKET_HANDLE64 socket_hd, sockaddr& server, int size_of_address)
{
    return connect(socket_hd, server, size_of_address);
}

int WinSocketWrapper::w_closesocket(SOCKET_HANDLE64 socket_hd)
{
    return closesocket(socket_hd);
}

int WinSocketWrapper::w_send(SOCKET_HANDLE64 socket_hd, uchar& arr[], int szToSend, int dd)
{
    return send(socket_hd, arr, szToSend, dd);
}

int WinSocketWrapper::w_recv(SOCKET_HANDLE64 socket_hd, uchar& arr[], int ReceiveBufferSize, int dd)
{
    return recv(socket_hd, arr, ReceiveBufferSize, dd);
}

int WinSocketWrapper::w_ioctlsocket(SOCKET_HANDLE64 socket_hd, uint fio, uint& is_non_block)
{
    return ioctlsocket(socket_hd, fio, is_non_block);
}

int WinSocketWrapper::w_bind(SOCKET_HANDLE64 socket_hd, sockaddr& server, int size_of_address)
{
    return bind(socket_hd, server, size_of_address);
}

int WinSocketWrapper::w_listen(SOCKET_HANDLE64 socket_hd, int client_count)
{
    return listen(socket_hd, client_count);
}

SOCKET_HANDLE64 WinSocketWrapper::w_accept(SOCKET_HANDLE64 socket_hd, int dd_1, int dd_2)
{
    return accept(socket_hd, dd_1, dd_2);
}

int WinSocketWrapper::w_WSAAsyncSelect(SOCKET_HANDLE64 socket_hd, long win_hd, uint key, int event)
{
    return  WSAAsyncSelect(socket_hd, win_hd, key, event);
}

int WinSocketWrapper::w_shutdown(SOCKET_HANDLE64 socket_hd, int dd)
{
    return shutdown(socket_hd, dd);
}

//
uint WinSocketWrapper::w_gethostbyname(uchar& arr[])
{
    return gethostbyname(arr);
}

ulong WinSocketWrapper::w_gethostbyname(char& arr[])
{
    return gethostbyname(arr);
}

//
uint WinSocketWrapper::w_inet_addr(uchar& arr[])
{
    return inet_addr(arr);
}

int WinSocketWrapper::w_WSAGetLastError()
{
    return WSAGetLastError();
}

uint WinSocketWrapper::w_htonl(uint dd)
{
    return htonl(dd);
}

ushort WinSocketWrapper::w_htons(ushort dd)
{
    return htons(dd);
}

//
void WinSocketWrapper::w_RtlMoveMemory(uint& ptr, uint start, int len)
{
    RtlMoveMemory(ptr, start, len);
}

void WinSocketWrapper::w_RtlMoveMemory(ushort& ptr, uint start, int len)
{
    RtlMoveMemory(ptr, start, len);
}

void WinSocketWrapper::w_RtlMoveMemory(ulong& ptr, ulong start, int len)
{
    RtlMoveMemory(ptr, start, len);
}

void WinSocketWrapper::w_RtlMoveMemory(ushort& ptr, ulong start, int len)
{
    RtlMoveMemory(ptr, start, len);
}
